package de.ozgcloud.common.binaryfile;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardCopyOption;

import org.apache.commons.io.FileUtils;

import de.ozgcloud.common.errorhandling.TechnicalException;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
public class TempFileUtils {
	static final String TMP_FILE_PREFIX = "filecached-inputstream";
	static final String TMP_FILE_SUFFIX = ".ozg-cloud.tmp";

	private static final String ERROR_MSG = "Error writing file to temp file.";

	public static File writeTmpFile(InputStream contentStream) {
		try {
			var tmpFile = createTmpFile();

			Files.copy(contentStream, tmpFile, StandardCopyOption.REPLACE_EXISTING);
			contentStream.close();
			return tmpFile.toFile();
		} catch (IOException e) {
			throw new TechnicalException(ERROR_MSG, e);
		}
	}

	public static File writeTmpFile(String stringData) {
		try {
			Path tmpFile = createTmpFile();

			try (var writer = new FileWriter(tmpFile.toFile())) {
				writer.write(stringData);
				writer.flush();
			}

			return tmpFile.toFile();
		} catch (IOException e) {
			throw new TechnicalException(ERROR_MSG, e);
		}
	}

	public static File writeTmpFile(byte[] data) {
		try {
			Path tmpFile = createTmpFile();

			FileUtils.writeByteArrayToFile(tmpFile.toFile(), data);

			return tmpFile.toFile();
		} catch (IOException e) {
			throw new TechnicalException(ERROR_MSG, e);
		}

	}

	public static Path createTmpFile() {
		try {
			var tmpFile = Files.createTempFile(TMP_FILE_PREFIX, TMP_FILE_SUFFIX);
			tmpFile.toFile().deleteOnExit();
			return tmpFile;
		} catch (IOException e) {
			throw new TechnicalException("Error creating temporary file.", e);
		}

	}
}
